/***************************************************************************
 *   Copyright (C) 2008 by S. MANKOWSKI / G. DE BURE skrooge@mankowski.fr  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
 ***************************************************************************/
/** @file
 * This file is Skrooge plugin for bank management.
 *
 * @author Stephane MANKOWSKI / Guillaume DE BURE
 */
#include "skgbankplugin.h"
#include "skgbankpluginwidget.h"
#include "skgbankboardwidget.h"
#include "skgaccountboardwidget.h"
#include "skgbank_settings.h"
#include "skgmainpanel.h"
#include "skgaccountobject.h"
#include "skgtraces.h"
#include "skgdocumentbank.h"
#include "skgtransactionmng.h"
#include "skgbankobject.h"

#include <kactioncollection.h>

/**
 * This plugin factory.
 */
K_PLUGIN_FACTORY(SKGBankPluginFactory, registerPlugin<SKGBankPlugin>();)
/**
 * This plugin export.
 */
K_EXPORT_PLUGIN(SKGBankPluginFactory("skrooge_bank", "skrooge_bank"))

SKGBankPlugin::SKGBankPlugin(QObject* iParent, const QVariantList& /*iArg*/)
    : SKGInterfacePlugin(iParent), m_currentBankDocument(NULL)
{
    SKGTRACEIN(10, "SKGBankPlugin::SKGBankPlugin");
}

SKGBankPlugin::~SKGBankPlugin()
{
    SKGTRACEIN(10, "SKGBankPlugin::~SKGBankPlugin");
    m_currentBankDocument = NULL;
    m_reconciliateAction = NULL;
}

bool SKGBankPlugin::setupActions(SKGDocument* iDocument, const QStringList& iArgument)
{
    SKGTRACEIN(10, "SKGBankPlugin::setupActions");
    Q_UNUSED(iArgument);
    m_currentBankDocument = qobject_cast<SKGDocumentBank*>(iDocument);
    if(m_currentBankDocument == NULL) return false;

    setComponentData(SKGBankPluginFactory::componentData());
    setXMLFile("skrooge_bank.rc");

    //Menu
    m_reconciliateAction = new KAction(KIcon("skrooge_duplicate"), i18nc("Verb", "Reconcile..."), this);
    connect(m_reconciliateAction, SIGNAL(triggered(bool)), this, SLOT(actionReconciliate()));
    actionCollection()->addAction(QLatin1String("edit_reconciliate"), m_reconciliateAction);
    m_reconciliateAction->setShortcut(Qt::ALT + Qt::Key_R);

    if(SKGMainPanel::getMainPanel()) SKGMainPanel::getMainPanel()->registedGlobalAction("edit_reconciliate", m_reconciliateAction);
    return true;
}

void SKGBankPlugin::close()
{
    SKGTRACEIN(10, "SKGBankPlugin::close");
}

int SKGBankPlugin::getNbDashboardWidgets()
{
    return 2;
}

QString SKGBankPlugin::getDashboardWidgetTitle(int iIndex)
{
    if(iIndex == 0) return i18nc("Noun, a list of items", "Accounts");
    else if(iIndex == 1) return i18nc("Noun, a list of items", "Banks");
    return "";
}

SKGWidget* SKGBankPlugin::getDashboardWidget(int iIndex)
{
    if(iIndex == 0) return new SKGAccountBoardWidget(m_currentBankDocument);
    else if(iIndex == 1) return new SKGBankBoardWidget(m_currentBankDocument);
    return NULL;
}

SKGTabPage* SKGBankPlugin::getWidget()
{
    SKGTRACEIN(10, "SKGBankPlugin::getWidget");
    return new SKGBankPluginWidget(m_currentBankDocument);
}

QWidget* SKGBankPlugin::getPreferenceWidget()
{
    SKGTRACEIN(10, "SKGBankPlugin::getPreferenceWidget");
    /*        QWidget* widget=new QWidget();
            ui.setupUi(widget);*/

    return NULL;
}

KConfigSkeleton* SKGBankPlugin::getPreferenceSkeleton()
{
    return skgbank_settings::self();
}

QString SKGBankPlugin::title() const
{
    return i18nc("Noun", "Bank and Account");
}

QString SKGBankPlugin::icon() const
{
    return "kwalletmanager";
}

QString SKGBankPlugin::toolTip() const
{
    return i18nc("Noun", "Bank and account management");
}

QStringList SKGBankPlugin::tips() const
{
    QStringList output;
    output.push_back(i18nc("Description of a tips", "<p>... you can associate a logo with your banks.</p>"));
    output.push_back(i18nc("Description of a tips", "<p>... accounts can be merged by drag & drop.</p>"));
    return output;
}

int SKGBankPlugin::getOrder() const
{
    //Must be one of the first
    return 10;
}

bool SKGBankPlugin::isInContext() const
{
    return true;
}

void SKGBankPlugin::refresh()
{
    SKGTRACEIN(10, "SKGBankPlugin::refresh");
    if(m_currentBankDocument && SKGMainPanel::getMainPanel()) {
        SKGObjectBase::SKGListSKGObjectBase selection = SKGMainPanel::getMainPanel()->getSelectedObjects();
        if(selection.count() > 0) {
            bool onAccount = (selection.at(0).getRealTable() == "account");
            if(m_reconciliateAction) m_reconciliateAction->setEnabled(onAccount);
        } else {
            if(m_reconciliateAction) m_reconciliateAction->setEnabled(false);
        }
    }
}

void SKGBankPlugin::actionReconciliate()
{
    if(m_currentBankDocument && SKGMainPanel::getMainPanel()) {
        //Open in operation plugin
        SKGObjectBase::SKGListSKGObjectBase selection = SKGMainPanel::getMainPanel()->getSelectedObjects();
        int nb = selection.count();
        for(int i = 0; i < nb; ++i) {
            SKGAccountObject accountObj = selection[i];

            QDomDocument doc("SKGML");
            doc.setContent(m_currentBankDocument->getParameter("SKGOPERATION_DEFAULT_PARAMETERS"));
            QDomElement root = doc.documentElement();
            if(root.isNull()) {
                root = doc.createElement("parameters");
                doc.appendChild(root);
            }

            root.setAttribute("account", accountObj.getName());
            root.setAttribute("modeInfoZone", "1");
            root.setAttribute("currentPage", "-1");
            SKGMainPanel::getMainPanel()->openPage(SKGMainPanel::getMainPanel()->getPluginByName("Skrooge operation plugin"), -1, doc.toString());
        }
    }
}

QList< SKGInterfacePlugin::SKGAdvice > SKGBankPlugin::advices() const
{
    SKGTRACEIN(10, "SKGBankPlugin::advices");
    QList< SKGInterfacePlugin::SKGAdvice > output;

    //Get bank without accounts
    SKGStringListList result;
    SKGServices::executeSelectSqliteOrder(m_currentBankDocument, "SELECT bank.t_name FROM bank WHERE NOT EXISTS (SELECT 1 FROM account WHERE account.rd_bank_id=bank.id)", result);
    int nb = result.count();
    for(int i = 1; i < nb; ++i) {  //Ignore header
        //Get parameters
        QStringList line = result.at(i);
        QString bank = line.at(0);

        SKGInterfacePlugin::SKGAdvice ad;
        ad.uuid = "skgbankplugin_withoutaccount|" + bank;
        ad.priority = 3;
        ad.shortMessage = i18nc("Advice on making the best (short)", "Bank '%1' has no account", bank);
        ad.longMessage = i18nc("Advice on making the best (long)", "Don't forget to remove useless banks");
        ad.autoCorrection.push_back(i18nc("Advice on making the best (action)", "Delete '%1'", bank));
        output.push_back(ad);
    }

    return output;
}

SKGError SKGBankPlugin::executeAdviceCorrection(const QString& iAdviceIdentifier, int iSolution) const
{
    if(m_currentBankDocument && iAdviceIdentifier.startsWith(QLatin1String("skgbankplugin_withoutaccount|"))) {
        //Get parameters
        QString bank = iAdviceIdentifier.right(iAdviceIdentifier.length() - 29);

        SKGError err;
        SKGBEGINTRANSACTION(*m_currentBankDocument, i18nc("Noun, name of the user action", "Delete unused bank")  , err);

        SKGBankObject bankObj(m_currentBankDocument);
        err = bankObj.setName(bank);
        if(err.isSucceeded()) err = bankObj.load();
        if(err.isSucceeded()) err = bankObj.remove();

        //status bar
        if(err.isSucceeded()) err = SKGError(0, i18nc("Successful message after an user action", "Unused bank deleted"));
        else err.addError(ERR_FAIL, i18nc("Error message", "Unused bank deletion failed"));


        //Display error
        SKGMainPanel::getMainPanel()->displayErrorMessage(err);

        return SKGError();
    }
    return SKGInterfacePlugin::executeAdviceCorrection(iAdviceIdentifier, iSolution);
}

#include "skgbankplugin.moc"
